import { useState, useEffect, useRef } from 'react';
import { Card, Row, Col, DatePicker, Select, Spin } from 'antd';
import ReactECharts from 'echarts-for-react';
import dayjs from 'dayjs';
import { api } from "@nice/client";

const { RangePicker } = DatePicker;
const { Option } = Select;

const DashboardPage = () => {
    const [dateRange, setDateRange] = useState<[dayjs.Dayjs, dayjs.Dayjs]>([
        dayjs().subtract(30, 'days'),
        dayjs()
    ]);
    const [loading, setLoading] = useState(true);


    // 使用useRef跟踪是否已执行过刷新
    const hasRefetched = useRef(false);

    // 获取网系类别数据
    const { data: systemTypeTerms, isLoading: loadingTypes, refetch: refetchSypes } = api.term.findMany.useQuery({
        where: {
            taxonomy: { slug: "system_type" },
            deletedAt: null,
        },
        orderBy: { order: "asc" },
    });

    // 获取设备故障数据
    const { data: devices, isLoading: loadingDevices, refetch: refetchDevices } = api.device.findMany.useQuery({
        where: {
            deletedAt: null,
            createdAt: {
                // 将开始日期设置为当天的开始时间 00:00:00
                gte: dateRange[0].startOf('day').toISOString(),
                // 将结束日期设置为当天的结束时间 23:59:59
                lte: dateRange[1].endOf('day').toISOString(),
            }
        },
        include: {
            department: true,
        },
    });
    useEffect(() => {
        // 只在数据加载完成后执行一次刷新
        if (!loadingTypes && !loadingDevices && !hasRefetched.current) {
            // 标记为已执行
            hasRefetched.current = true
            // 刷新数据
            refetchSypes();
            refetchDevices();
            setLoading(false);
        }
    }, [loadingTypes, loadingDevices]);


    // 处理日期范围变化
    const handleDateRangeChange = (dates, dateStrings) => {
        if (dates) {
            setDateRange([dates[0], dates[1]]);
        } else {
            setDateRange([dayjs().subtract(30, 'days'), dayjs()]);
        }
    };

    // 准备各个网系故障情况数据（按时间段）
    const prepareSystemFaultsByTimeData = () => {
        if (!devices || !systemTypeTerms) return { xAxis: [], series: [] };

        // 获取选择的日期范围
        const startDate = dateRange[0];
        const endDate = dateRange[1];
        const diffDays = endDate.diff(startDate, 'day');

        // 根据时间跨度选择合适的间隔
        const intervals = [];
        let format = '';

        if (diffDays <= 14) {
            // 小于等于两周，按天展示
            format = 'MM-DD';
            for (let i = 0; i <= diffDays; i++) {
                intervals.push(startDate.add(i, 'day'));
            }
        } else if (diffDays <= 90) {
            // 小于等于3个月，按周展示
            format = 'MM-DD';
            const weeks = Math.ceil(diffDays / 7);
            for (let i = 0; i < weeks; i++) {
                const weekStart = startDate.add(i * 7, 'day');
                intervals.push(weekStart);
            }
        } else {
            // 大于3个月，按月展示
            format = 'YYYY-MM';
            const months = Math.ceil(diffDays / 30);
            for (let i = 0; i < months; i++) {
                const monthStart = startDate.add(i, 'month').startOf('month');
                intervals.push(monthStart);
            }
        }

        // 格式化时间段显示
        const timeLabels = intervals.map(date => {
            if (diffDays <= 14) {
                return date.format(format);
            } else if (diffDays <= 90) {
                return `${date.format(format)}至${date.add(6, 'day').format(format)}`;
            } else {
                return date.format(format);
            }
        });

        // 统计每个网系在每个时间段的故障数
        const systemNames = systemTypeTerms.map(type => type.name);
        const systemIdMap = {};
        systemTypeTerms.forEach(type => {
            systemIdMap[type.id] = type.name;
        });

        const seriesData = systemNames.map(name => ({
            name,
            type: 'bar',
            data: new Array(intervals.length).fill(0)
        }));

        devices.forEach(device => {
            const systemName = systemIdMap[device.systemType] || '未知';
            const systemIndex = systemNames.indexOf(systemName);
            if (systemIndex !== -1) {
                const createDate = dayjs(device.createdAt);

                for (let i = 0; i < intervals.length; i++) {
                    const nextInterval = null;

                    if (diffDays <= 14) {
                        // 按天，检查是否是同一天
                        if (createDate.format('YYYY-MM-DD') === intervals[i].format('YYYY-MM-DD')) {
                            seriesData[systemIndex].data[i]++;
                        }
                    } else if (diffDays <= 90) {
                        // 按周，检查是否在这周内
                        const weekEnd = intervals[i].add(6, 'day').endOf('day');
                        if (createDate.isAfter(intervals[i]) && createDate.isBefore(weekEnd)) {
                            seriesData[systemIndex].data[i]++;
                        }
                    } else {
                        // 按月，检查是否在这个月内
                        const monthEnd = intervals[i].endOf('month');
                        if (createDate.isAfter(intervals[i]) && createDate.isBefore(monthEnd)) {
                            seriesData[systemIndex].data[i]++;
                        }
                    }
                }
            }
        });

        return {
            xAxis: timeLabels,
            series: seriesData
        };
    };

    // 准备各个网系故障率数据
    const prepareSystemFaultRateData = () => {
        if (!devices || !systemTypeTerms) return { names: [], values: [] };

        const systemCounts = {};
        const totalDevices = devices.length;
        console.log("devices", devices.length);

        // 初始化所有网系的计数
        systemTypeTerms.forEach(type => {
            systemCounts[type.name] = 0;
        });

        // 统计每个网系的故障数
        devices.forEach(device => {
            const systemType = systemTypeTerms?.find(t => t.id === device.systemType);
            if (systemType) {
                systemCounts[systemType.name] = (systemCounts[systemType.name] || 0) + 1;
            }
        });

        const names = Object.keys(systemCounts);
        const values = names.map(name => ({
            value: totalDevices ? ((systemCounts[name] / totalDevices) * 100).toFixed(2) : 0,
            name
        }));

        return { names, values };
    };

    // 准备故障处置完成率数据
    const prepareFaultCompletionRateData = () => {
        if (!devices) return { value: 0 };
        const completedFaults = devices.filter(device =>
            device.deviceStatus === 'normal'
        ).length;

        const totalFaults = devices.length;
        const completionRate = totalFaults ? (completedFaults / totalFaults) * 100 : 0;

        return { value: parseFloat(completionRate.toFixed(2)) };
    };

    // 网系故障时间段分布选项
    const getSystemFaultsByTimeOption = () => {
        const data = prepareSystemFaultsByTimeData();
        return {
            title: {
                text: '各网系故障时间分布',
                left: 'center'
            },
            tooltip: {
                trigger: 'axis',
                axisPointer: {
                    type: 'shadow'
                }
            },
            legend: {
                data: data.series.map(item => item.name),
                bottom: 10
            },
            grid: {
                left: '3%',
                right: '4%',
                bottom: '15%',
                top: '15%',
                containLabel: true
            },
            xAxis: {
                type: 'category',
                data: data.xAxis
            },
            yAxis: {
                type: 'value',
                name: '故障数量'
            },
            series: data.series
        };
    };

    // 网系故障率选项
    const getSystemFaultRateOption = () => {
        const data = prepareSystemFaultRateData();
        return {
            title: {
                text: '各网系故障率',
                left: 'center'
            },
            tooltip: {
                trigger: 'item',
                formatter: '{a} <br/>{b}: {c}%'
            },
            legend: {
                orient: 'vertical',
                left: 10,
                top: 'center',
                data: data.names
            },
            series: [
                {
                    name: '故障率',
                    type: 'pie',
                    radius: ['40%', '70%'],
                    avoidLabelOverlap: false,
                    itemStyle: {
                        borderRadius: 10,
                        borderColor: '#fff',
                        borderWidth: 2
                    },
                    label: {
                        show: true,
                        formatter: '{b}: {c}%'
                    },
                    emphasis: {
                        label: {
                            show: true,
                            fontSize: '15',
                            fontWeight: 'bold'
                        }
                    },
                    data: data.values
                }
            ]
        };
    };

    // 故障处置完成率选项
    const getFaultCompletionRateOption = () => {
        const data = prepareFaultCompletionRateData();
        return {
            title: {
                text: '故障处置完成率',
                left: 'center'
            },
            tooltip: {
                formatter: '{a} <br/>{b} : {c}%'
            },
            series: [
                {
                    name: '完成率',
                    type: 'gauge',
                    detail: { formatter: '{value}%' },
                    data: [{ value: data.value, name: '完成率' }],
                    axisLine: {
                        lineStyle: {
                            width: 30,
                            color: [
                                [0.3, '#ff6e76'],
                                [0.7, '#fddd60'],
                                [1, '#7cffb2']
                            ]
                        }
                    },
                    pointer: {
                        itemStyle: {
                            color: 'auto'
                        }
                    }
                }
            ]
        };
    };

    return (
        <div className="p-4 min-h-screen bg-gray-50">
            <h1 className="text-2xl font-bold mb-6">故障数据可视化面板</h1>
            {/* <RangePicker
                value={dateRange}
                onChange={handleDateRangeChange}
                format="YYYY-MM-DD"
                className="mb-2"
            /> */}

            <Row className="mb-4">
                <Col span={24}>
                    <Card className="shadow-sm">
                        <div className="flex justify-between items-center ">
                            <span className="text-lg font-medium">数据筛选</span>
                            <RangePicker
                                value={dateRange}
                                onChange={handleDateRangeChange}
                                format="YYYY-MM-DD"
                            />
                        </div>
                    </Card>
                </Col>
            </Row>

            {loading ? (
                <div className="flex justify-center items-center h-64">
                    <Spin size="large" tip="数据加载中..." />
                </div>
            ) : (
                <Row gutter={[16, 16]}>
                    <Col span={12}>
                        <Card className="shadow-sm hover:shadow-md transition-shadow duration-300">
                            <ReactECharts
                                option={getFaultCompletionRateOption()}
                                style={{ height: '400px' }}
                            />
                        </Card>
                    </Col>
                    <Col span={12}>
                        <Card className="shadow-sm hover:shadow-md transition-shadow duration-300">
                            <ReactECharts
                                option={getSystemFaultRateOption()}
                                style={{ height: '400px' }}
                            />
                        </Card>
                    </Col>

                    <Col span={24}>
                        <Card className="shadow-sm hover:shadow-md transition-shadow duration-300">
                            <ReactECharts
                                option={getSystemFaultsByTimeOption()}
                                style={{ height: '450px' }}
                            />
                        </Card>
                    </Col>
                </Row>
            )}
        </div>
    );
};

export default DashboardPage;