import { Button, Checkbox, Modal, Table, Upload, Tag } from "antd"; // 添加 Tag 导入
import { ColumnsType } from "antd/es/table";
import { api, useDevice, useStaff } from "@nice/client";
import { useEffect, useState, useImperativeHandle, forwardRef, useRef } from "react";
import toast from "react-hot-toast";
import React from "react";
import { useMainContext } from "../../layout/MainProvider";
import { ExclamationCircleOutlined, ImportOutlined, ExportOutlined } from "@ant-design/icons";
import dayjs from "dayjs";
import { utils, writeFile, read } from 'xlsx';

// 提取处理嵌套字段的函数
const getNestedValue = (record: any, dataIndex: string | string[]) => {
  if (Array.isArray(dataIndex)) {
    return dataIndex.reduce((obj, key) => obj?.[key], record);
  }
  return record[dataIndex];
};

// 添加props接口
interface DeviceTableProps {
  onSelectedChange?: (keys: React.Key[], data: any[]) => void;
}

// 使用forwardRef包裹组件
const DeviceTable = forwardRef(({ onSelectedChange }: DeviceTableProps, ref) => {
  const { form, setVisible, searchValue, formValue, editingRecord, setEditingRecord } = useMainContext();

  const { create } = useDevice();
  const [descModalVisible, setDescModalVisible] = useState(false);
  const [currentDesc, setCurrentDesc] = useState({ title: "", desc: "" });

  // 处理点击故障名称显示描述信息
  const handleShowDesc = (record) => {
    setCurrentDesc({
      title: record.showname || "未知故障",
      desc: record.notes || "无故障详情"
    });
    setDescModalVisible(true);
  };

  const { data: devices, isLoading, refetch } = api.device.findMany.useQuery(
    {
      where: (searchValue as any) || { deletedAt: null },
      include: {
        department: true,
      },
      orderBy: { createdAt: "desc" },
    },
    {
      enabled: true,
    }
  );
  console.log("devices",devices);
  
  const { data: systemTypeTerms, refetch: refetchSystemType } =
    api.term.findMany.useQuery({
      where: {
        taxonomy: { slug: "system_type" },
        deletedAt: null,
      },
      include: {
        children: true,
      },
      orderBy: { order: "asc" },
    });
  const { data: deviceTypeTerms, refetch: refetchDeviceType } = api.term.findMany.useQuery({
    where: {
      taxonomy: { slug: 'device_type' },
      deletedAt: null,
    }
  });
  const { mutate: softDeleteByIds } = api.device.softDeleteByIds.useMutation();
  useEffect(() => {
    // console.log(devices);
    // refetch();
    refetchSystemType();
    refetchDeviceType();
  }, [devices]);
  // const { softDeleteByIds } = useStaff()

  // 添加部门数据查询
  const { data: departments } = api.department.findMany.useQuery({
    where: {
      deletedAt: null,
    }
  }, {
    enabled: true, // 确保此查询被执行
  });

  const getTermNameById = (termId, termType) => {
    if (!termId) return "未知";

    const terms = termType === 'system_type' ? systemTypeTerms : deviceTypeTerms;
    const term = terms?.find(t => t.id === termId);
    return term?.name || "未知";
  };
  const handleDelete = (record) => {
    Modal.confirm({
      title: "确认删除",
      icon: <ExclamationCircleOutlined />,
      content: `确定要删除故障 "${record.showname || "未命名故障"}" 吗？`,
      okText: "删除",
      okType: "danger",
      cancelText: "取消",
      onOk() {
        softDeleteByIds(
          {
            ids: [record.id],
          },
          {
            onSuccess: () => {
              toast.success("删除成功");
              refetch();
            },
            onError: (error) => {
              console.error("删除故障时出错:", error);
              toast.error("删除失败");
            },
          }
        );
      },
    });
  };

  const columns: ColumnsType<any> = [
    {
      title: "网系类别",
      dataIndex: "systemType",
      key: "systemType",
      align: "center",
      render: (text, record) => {
        return getTermNameById(record.systemType, 'system_type');
      },
    },
    {
      title: "故障类型",
      dataIndex: "deviceType",
      key: "deviceType",
      align: "center",
      render: (text, record) => {
        return getTermNameById(record.deviceType, 'device_type');
      },
    },
    {
      title: "单位",
      dataIndex: "deptId",
      key: "deptId",
      align: "center",
      render: (text, record) => {
        // 如果有部门关联，显示部门名称
        if (record.deptId && (record as any)?.department?.name) {
          return (record as any)?.department?.name;
        }
        // 否则显示responsiblePerson（如果存在）
        if (record.responsiblePerson) {
          return record.responsiblePerson;
        }
        // 最后才显示未知
        return "未知";
      },
    },
    {
      title: "故障名称",
      dataIndex: "showname",
      key: "showname",
      align: "center",
      render: (text, record) => (
        <div onClick={() => handleShowDesc(record)}
          style={{
            cursor: 'pointer',
            padding: '8px 0',
            // backgroundColor: 'black',  
            fontWeight: 'bold',
          }}>
          {text || "未命名故障"}
        </div>
      ),
    },
    {
      title: "故障状态",
      dataIndex: "deviceStatus",
      key: "deviceStatus",
      align: "center",
      render: (status) => {
        const statusConfig = {
          normal: {
            text: "已修复",
            color: "success"
          },
          maintenance: {
            text: "维修中",
            color: "processing"
          },
          broken: {
            text: "未修复",
            color: "error"
          }
        };

        const config = statusConfig[status] || {
          text: "未知",
          color: "default"
        };

        return (
          <Tag color={config.color} style={{ minWidth: '60px' , textAlign: 'center' }}>
            {config.text}
          </Tag>
        );
      }
    },
    {
      title: "时间",
      dataIndex: "createdAt",
      key: "createdAt",
      align: "center",
      render: (text, record) => record.createdAt ? dayjs(record.createdAt).format('YYYY-MM-DD HH:mm:ss') : "未知",
    },
    {
      title: "操作",
      key: "action",
      align: "center",
      render: (_, record) => (
        <div className="flex space-x-2 justify-center">
          <Button
            type="primary"
            key={record.id}
            onClick={() => handleEdit(record)}

          >
            编辑
          </Button>
          <Button
            danger
            onClick={() => handleDelete(record)}

          >
            删除
          </Button>
        </div>
      ),
    },
  ];

  useEffect(() => {
    if (editingRecord) {
      form.setFieldsValue(editingRecord);
      console.log(editingRecord);
    }
  }, [editingRecord]);

  const handleEdit = (record) => {
    setEditingRecord(record);
    form.setFieldsValue(record);
    setVisible(true);
  };

  const [selectedRowKeys, setSelectedRowKeys] = useState<React.Key[]>([]);

  const onSelectChange = (newSelectedRowKeys: React.Key[], selectedRows: any[]) => {
    setSelectedRowKeys(newSelectedRowKeys);
  };
  //导出函数 
  const handleExportSelected = () => {
    // 获取选中的行数据
    const selectedData = devices?.filter(item =>
      selectedRowKeys.includes(item.id)
    );

    if (!selectedData || selectedData.length === 0) {
      toast.error("没有选中任何数据");
      return;
    }

    try {
      // 格式化导出数据
      const exportData = selectedData.map(item => ({
        '网系类别': getTermNameById(item.systemType, 'system_type'),
        '故障类型': getTermNameById(item.deviceType, 'device_type'),
        '单位': (item as any)?.department?.name || "未知",
        '故障名称': item?.showname || "未命名故障",
        '故障状态': (() => {
          const statusMap = {
            normal: "已修复",
            maintenance: "维修中",
            broken: "损坏",
            idle: "闲置"
          };
          return statusMap[item.deviceStatus] || "未知";
        })(),
        '时间': item.createdAt ? dayjs(item.createdAt).format('YYYY-MM-DD') : "未知",
        '描述': item.notes || "无描述"
      }));

      // 创建工作簿
      const wb = utils.book_new();
      const ws = utils.json_to_sheet(exportData);
      utils.book_append_sheet(wb, ws, "故障数据");

      // 导出Excel文件
      writeFile(wb, `故障数据_${dayjs().format('YYYYMMDD_HHm  mss')}.xlsx`);

      toast.success(`成功导出 ${selectedData.length} 条数据`);
    } catch (error) {
      console.error("导出数据错误:", error);
      toast.error("导出失败");
    }
  };

  // 添加用于文件上传的ref
  const uploadRef = useRef<HTMLInputElement>(null);

  // 处理文件选择
  const handleFileChange = async (e: React.ChangeEvent<HTMLInputElement>) => {
    const files = e.target.files;
    if (!files || files.length === 0) return;

    const file = files[0];
    try {
      // 读取Excel/CSV文件
      const data = await file.arrayBuffer();
      const workbook = read(data);
      const worksheet = workbook.Sheets[workbook.SheetNames[0]];
      const jsonData = utils.sheet_to_json(worksheet);

      // 转换为符合系统格式的数据 
      const records = jsonData.map((row: any) => {
        const unitName = row['单位'] ? String(row['单位']) : "";
        return {
          systemType: findTermIdByName(row['网系类别'], 'system_type'),
          deviceType: findTermIdByName(row['故障类型'], 'device_type'),
          showname: row['故障名称'] ? String(row['故障名称']) : "",
          // 尝试关联部门ID
          deptId: unitName ? findDeptIdByName(unitName) : null,
          // 同时保存原始单位名称
          responsiblePerson: unitName,
          deviceStatus: getStatusKeyByValue(row['故障状态']),
          notes: row['描述'] ? String(row['描述']) : ""
        };
      });
      // 确认是否有有效数据
      if (records.length === 0) {
        toast.error("未找到有效数据");
        return;
      }
      // 批量创建记录
      await batchImportRecords(records);
      refetch(); // 刷新表格数据
    } catch (error) {
      console.error("导入失败:", error);
      toast.error("导入失败，请检查文件格式");
    } finally {
      // 清空文件输入，允许再次选择同一文件
      if (uploadRef.current) uploadRef.current.value = '';
    }
  };

  // 查找术语ID
  const findTermIdByName = (name: string, type: 'system_type' | 'device_type') => {
    if (!name) return null;

    const terms = type === 'system_type' ? systemTypeTerms : deviceTypeTerms;
    const term = terms?.find(t => t.name === name);
    return term?.id || null;
  };

  // 改进查找部门ID函数
  const findDeptIdByName = (name: string) => {
    if (!name || name === "未知") return null;

    // 直接从部门数据中查找
    const department = departments?.find(dept => dept.name === name);
    if (department) {
      return department.id;
    }

    // 备用：尝试从设备中查找
    const matchedDevice = devices?.find(device =>
      (device as any)?.department?.name === name
    );

    return matchedDevice?.deptId || null;
  };

  // 获取状态键
  const getStatusKeyByValue = (value: string) => {
    const statusMap: Record<string, string> = {
      "已修复": "normal",
      "维修中": "maintenance",
      "未修复 ": "broken",
    };
    return statusMap[value] || "normal";
  };

  // 改进批量导入记录函数
  // 改进批量导入记录函数，使用分批处理
  const batchImportRecords = async (records: any[]) => {
    if (!records || records.length === 0) return;
    try {
      // 过滤出有效记录
      const validRecords = records.filter(record =>
        record.systemType || record.deviceType || record.showname
      );

      if (validRecords.length === 0) {
        toast.error("没有找到有效的记录数据");
        return;
      }
      // 设置批处理大小
      const batchSize = 5;
      let successCount = 0;
      let totalProcessed = 0;
      // 显示进度提示
      const loadingToast = toast.loading(`正在导入数据...`);
      // 分批处理数据
      for (let i = 0; i < validRecords.length; i += batchSize) {
        const batch = validRecords.slice(i, i + batchSize);

        // 串行处理每一批数据
        for (const record of batch) {
          try {
            await create.mutateAsync(record);
            successCount++;
          } catch (error) {
            console.error(`导入记录失败: ${record.showname || "未命名"}`, error);
          }
        }

        totalProcessed += batch.length;
        // 更新导入进度
        toast.loading(`已处理 ${totalProcessed}/${validRecords.length} 条数据...`,
          { id: loadingToast });
      }

      toast.dismiss(loadingToast);

      // 显示结果
      if (successCount === validRecords.length) {
        toast.success(`成功导入 ${successCount} 条数据`);
      } else {
        toast.success(`成功导入 ${successCount}/${validRecords.length} 条数据，部分记录导入失败`);
      }

      // 刷新数据
      refetch();
    } catch (error) {
      console.error("批量导入失败:", error);
      toast.error("导入过程中发生错误");
    }
  };

  // 添加导出模板功能
  const handleExportTemplate = () => {
    try {
      // 创建一个示例记录
      const templateData = [
        {
          '网系类别': systemTypeTerms?.[0]?.name || '网系类别1',
          '故障类型': deviceTypeTerms?.[0]?.name || '故障类型1',
          '单位': '单位名称',
          '故障名称': '示例故障名称',
          '故障状态': '未修复', // 可选值:已修复, 维修中, 未修复
          '描述': '这是一个示例描述'
        }
      ];

      // 创建工作簿
      const wb = utils.book_new();
      const ws = utils.json_to_sheet(templateData);
      utils.book_append_sheet(wb, ws, "导入模板");

      // 导出Excel文件
      writeFile(wb, `故障导入模板.xlsx`);

      toast.success("已下载导入模板");
    } catch (error) {
      console.error("导出模板失败:", error);
      toast.error("下载模板失败");
    }
  };

  // 触发文件选择
  const handleImportClick = () => {
    if (uploadRef.current) {
      uploadRef.current.click();
    }
  };

  const rowSelection = {
    selectedRowKeys,
    onChange: onSelectChange,
    columnWidth: 70,
    columnTitle: (
      <div className="flex items-center justify-center">
        <Checkbox
          indeterminate={selectedRowKeys?.length > 0 && selectedRowKeys?.length < (devices?.length || 0)}
         checked={(devices?.length || 0) > 0 && selectedRowKeys?.length === (devices?.length || 0)}
          onChange={(e) => {
            const checked = e.target.checked;
            const newSelectedRowKeys = checked ? (devices || []).map(item => item.id) : [];
            onSelectChange(newSelectedRowKeys, checked ? devices : []);
          }}
        />
        <span className="ml-1.5 text-xs whitespace-nowrap">全选</span>
      </div>
    ),
    preserveSelectedRowKeys: true // 这个属性保证翻页时选中状态不丢失
  
  }
 
  
  const TableHeader = () => (
    <div className="w-full flex justify-between mb-2">
      <span>
      </span>
      <div className="flex space-x-2">
        <input
          type="file"
          ref={uploadRef}
          onChange={handleFileChange}
          style={{ display: 'none' }}
          accept=".xlsx,.xls,.csv"
        />
        <Button
          icon={<ImportOutlined />}
          type="primary"
          onClick={handleExportTemplate}
        >
          下载模板文件
        </Button>
        <Button
          icon={<ImportOutlined />}
          type="primary"
          onClick={handleImportClick}
        >
          导入
        </Button>
        <Button
          type="primary"
          onClick={handleExportSelected}
          disabled={!selectedRowKeys || selectedRowKeys.length === 0}
          icon={<ExportOutlined />}
        >
          {selectedRowKeys?.length > 0 ? `导出 (${selectedRowKeys.length})项数据` : '导出选中数据'}
        </Button>
      </div>
    </div>
  );

  return (
    <>
      <TableHeader />
      <Table
        rowSelection={rowSelection}
        rowKey="id"
        columns={columns}
        dataSource={devices}
        size="middle"
        tableLayout="fixed"
        rowClassName={(record, index) =>
          index % 2 === 0 ? "bg-white hover:bg-blue-100" : "bg-gray-100 hover:bg-blue-100"
        }
        onRow={(record) => {
          return {
            style: { cursor: 'pointer' },
            onMouseEnter: () => { }, // 触发悬停效果
          };
        }}
        onHeaderRow={() => {
          return {
            style: {
              backgroundColor: '#d6e4ff',
            },
          };
        }}
        scroll={{ x: "max-content" }}
        pagination={{
          position: ["bottomCenter"],
          className: "flex justify-center mt-4",
          pageSize: 10,
          showSizeChanger: true,
          pageSizeOptions: ["10", "20", "30"],
          hideOnSinglePage: true,
          responsive: true,
          showTotal: (total, range) => `共${total} 条数据`,
          showQuickJumper: true,
        }}
        components={{
          header: {
            cell: (props) => (
              <th
                {...props}
                style={{
                  whiteSpace: "nowrap",
                  overflow: "hidden",
                  textOverflow: "ellipsis",
                  textAlign: "center",
                }}
              />
            ),
          },
        }}
        bordered
        className="device-table-no-wrap"
      />
      <Modal
        title={currentDesc.title}
        open={descModalVisible}
        onCancel={() => setDescModalVisible(false)}
        footer={[
          <Button key="close" onClick={() => setDescModalVisible(false)}>
            关闭
          </Button>
        ]}
      >
        <div className="py-4">
          <div className="text-lg font-medium mb-2">故障详情</div>
          <div className="bg-gray-50 p-4 rounded-md whitespace-pre-wrap">
            {currentDesc.desc}
          </div>
        </div>
      </Modal>
    </>
  );
});

export default DeviceTable;

